#include "RenderTest.h"
#include "RenderTestStyle.h"
#include "RenderTestCommands.h"
#include "LevelEditor.h"
#include "Widgets/Docking/SDockTab.h"
#include "Widgets/Layout/SBox.h"
#include "Widgets/Text/STextBlock.h"
#include "Framework/MultiBox/MultiBoxBuilder.h"
#include "Widgets/SViewport.h"
#include "Slate/SceneViewport.h"
#include "RenderTestViewportClient.h"

static const FName RenderTestTabName("RenderTest");

#define LOCTEXT_NAMESPACE "FRenderTestModule"

void FRenderTestModule::StartupModule()
{
	// This code will execute after your module is loaded into memory; the exact timing is specified in the .uplugin file per-module
	
	FRenderTestStyle::Initialize();
	FRenderTestStyle::ReloadTextures();

	FRenderTestCommands::Register();
	
	PluginCommands = MakeShareable(new FUICommandList);

	PluginCommands->MapAction(
		FRenderTestCommands::Get().OpenPluginWindow,
		FExecuteAction::CreateRaw(this, &FRenderTestModule::PluginButtonClicked),
		FCanExecuteAction());
		
	FLevelEditorModule& LevelEditorModule = FModuleManager::LoadModuleChecked<FLevelEditorModule>("LevelEditor");
	
	{
		TSharedPtr<FExtender> MenuExtender = MakeShareable(new FExtender());
		MenuExtender->AddMenuExtension("WindowLayout", EExtensionHook::After, PluginCommands, FMenuExtensionDelegate::CreateRaw(this, &FRenderTestModule::AddMenuExtension));

		LevelEditorModule.GetMenuExtensibilityManager()->AddExtender(MenuExtender);
	}
	
	{
		TSharedPtr<FExtender> ToolbarExtender = MakeShareable(new FExtender);
		ToolbarExtender->AddToolBarExtension("Settings", EExtensionHook::After, PluginCommands, FToolBarExtensionDelegate::CreateRaw(this, &FRenderTestModule::AddToolbarExtension));
		
		LevelEditorModule.GetToolBarExtensibilityManager()->AddExtender(ToolbarExtender);
	}
	
	FGlobalTabmanager::Get()->RegisterNomadTabSpawner(RenderTestTabName, FOnSpawnTab::CreateRaw(this, &FRenderTestModule::OnSpawnPluginTab))
		.SetDisplayName(LOCTEXT("FRenderTestTabTitle", "RenderTest"))
		.SetMenuType(ETabSpawnerMenuType::Hidden);
}

void FRenderTestModule::ShutdownModule()
{
	// This function may be called during shutdown to clean up your module.  For modules that support dynamic reloading,
	// we call this function before unloading the module.
	FRenderTestStyle::Shutdown();

	FRenderTestCommands::Unregister();

	FGlobalTabmanager::Get()->UnregisterNomadTabSpawner(RenderTestTabName);
}

TSharedRef<SDockTab> FRenderTestModule::OnSpawnPluginTab(const FSpawnTabArgs& SpawnTabArgs)
{
	TSharedRef<SDockTab> Tab = SNew(SDockTab)
		.TabRole(ETabRole::NomadTab)
		[
			// Put your tab content here!
			SNew(SBox)
			.HAlign(HAlign_Fill)
		.VAlign(VAlign_Fill)
		[
			SAssignNew(ViewportWidget, SViewport)
		]
		];

	ViewportClient = MakeShareable(new FRenderTestViewportClient());
	Viewport = MakeShareable(new FSceneViewport(ViewportClient.Get(), ViewportWidget));

	ViewportWidget->SetViewportInterface(Viewport.ToSharedRef());

	return Tab;
}

void FRenderTestModule::PluginButtonClicked()
{
	FGlobalTabmanager::Get()->InvokeTab(RenderTestTabName);
}

void FRenderTestModule::AddMenuExtension(FMenuBuilder& Builder)
{
	Builder.AddMenuEntry(FRenderTestCommands::Get().OpenPluginWindow);
}

void FRenderTestModule::AddToolbarExtension(FToolBarBuilder& Builder)
{
	Builder.AddToolBarButton(FRenderTestCommands::Get().OpenPluginWindow);
}

#undef LOCTEXT_NAMESPACE
	
IMPLEMENT_MODULE(FRenderTestModule, RenderTest)